// Trigonometric Grapher for Win95

#include <windows.h>
#include <math.h>
#include <bwcc.h>

#include "trigfunc.h"

#define NAMELEN 50

typedef struct tagGridProperties
{
  char Name[15];
  BOOL Grid;
  int Connect;
  int Graph;
  BOOL Sonar;
  COLORREF Color[8];
  double Modifier[NUM_MODIFIERS];
  char ModifierStr[NUM_MODIFIERS][NAMELEN];
} GridProperties;

HMENU hMenu;
HBITMAP hGraph;
RECT GraphRect;
BOOL GraphUpdated;
HWND hPrintDlg;
PRINTDLG pds;
BOOL ThreadRunning;
HANDLE hThread;
HINSTANCE ghInstance;
BOOL Canceled;
int NextPlot;
GridProperties gp;

const char ClassName[] = "Trigonometric Function Class - version 1.6 for Win32";

BOOL CALLBACK AboutDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK PrintDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK TrigWndProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK PropertiesDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL GetTrigValue(double x, double *y, int Method, double *Modifier);
void SetGraphWindowText(HWND hWnd, LPSTR Text);
void DrawAsymptote(HDC hDC, double x, int w, int h, int XOrgin, GridProperties *g);
void DrawGraph(HDC hDC, RECT ClientRect, GridProperties *g);
void DrawGraphWithLabels(HDC hDC, RECT ClientRect, GridProperties *g);
void ChangeGraphMenu(int OldMenu, int NewMenu, LPSTR Name);
void ChangeMenuFlag(int MenuItem, BOOL *Flag, HWND hWnd);
int GraphXToClient(double GraphPoint, int ClientWidth, GridProperties *g);
int GraphYToClient(double GraphPoint, int ClientHeight, GridProperties *g);
double ClientToGraphX(int GraphPoint, int ClientWidth, GridProperties *g);
double ClientToGraphY(int GraphPoint, int ClientHeight, GridProperties *g);
double round(double x, int p);
int PrintGraph(GridProperties *g);
void SavePrefs(GridProperties *g, HWND hWnd);
void LoadPrefs(GridProperties *g, HWND hWnd);
void SaveCustomColors(DWORD *g);
void LoadCustomColors(DWORD *g);
void CalculateModifiers(GridProperties *g);
int StrangeRound(double x, int p);

#pragma argsused
int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
  HWND hWnd;
  MSG msg;
  HACCEL hAccel;

  BWCCRegister(hInstance);
  ghInstance = hInstance;
  hGraph = 0;
  GraphRect.left = 0;
  GraphRect.right = 0;
  GraphRect.top = 0;
  GraphRect.bottom = 0;
  GraphUpdated = FALSE;

  // Create a Window Class
  if (!hPrevInstance)
  {
	 WNDCLASS wc;

	 // Main Window
	 wc.style = CS_HREDRAW | CS_VREDRAW;
	 wc.lpfnWndProc = (WNDPROC) TrigWndProc;
	 wc.cbClsExtra = 0;
	 wc.cbWndExtra = 0;
	 wc.hInstance = hInstance;
	 wc.hIcon = LoadIcon(hInstance, "SINE_ZZZICON");
	 wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	 wc.hbrBackground = CreateSolidBrush(RGB(255, 255, 255));
	 wc.lpszMenuName = "GRAPH";
	 wc.lpszClassName = ClassName;
	 if (!RegisterClass(&wc))
		return 0;
  }

  // Create the main Window
  hWnd = CreateWindow(ClassName, "Trigonometric Grapher", WS_OVERLAPPEDWINDOW
					| WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
					CW_USEDEFAULT, HWND_DESKTOP, NULL, hInstance, NULL);
  hAccel = LoadAccelerators(hInstance, "GRAPH");
  ShowWindow(hWnd, nCmdShow);
  UpdateWindow(hWnd);

  while (GetMessage(&msg, NULL, 0, 0))
  {
	 if (!TranslateAccelerator(hWnd, hAccel, &msg))
	 {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	 }
  }

  if (pds.hDevMode != NULL)
  {
	 GlobalFree(pds.hDevMode);
	 GlobalFree(pds.hDevNames);
  }
  if (!hThread)
	 CloseHandle(hThread);

  // Unregister the class
  UnregisterClass(ClassName, hInstance);

  // Delete the bitmap, if any
  if (hGraph != 0)
	 DeleteObject(hGraph);

  // Exit the program
  return 0;
}

#pragma argsused
BOOL CALLBACK TrigWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
	 case WM_CREATE :
	 {
		// Initialize menu
		ZeroMemory(gp.Name, sizeof(char) * 15);
		hMenu = GetMenu(hWnd);
		hPrintDlg = NULL;

		// Get the stored values
		LoadPrefs(&gp, hWnd);

		// Initialize pds Structure
		ZeroMemory(&pds, sizeof(PRINTDLG));
		pds.lStructSize = sizeof(PRINTDLG);
		pds.hDevMode = NULL;
		pds.hDevNames = NULL;

		// Initialize Thread values
		hThread = NULL;
		ThreadRunning = FALSE;
		return TRUE;
	 }
	 case WM_COMMAND :
	 {
		switch (LOWORD(wParam))
		{
		  case CM_GRAPHCOSINE: case CM_GRAPHTANGENT: case CM_GRAPHCOSECANT:
		  case CM_GRAPHSECANT: case CM_GRAPHCOTANGENT: case CM_GRAPHSINE:
		  {
			 ChangeGraphMenu(gp.Graph, LOWORD(wParam), gp.Name);
			 gp.Graph = LOWORD(wParam);
			 SetGraphWindowText(hWnd, gp.Name);
			 InvalidateRect(hWnd, NULL, TRUE);
			 GraphUpdated = TRUE;
			 break;
		  }

		  case CM_GRAPHEXIT:
		  {
			 if (ThreadRunning)
			 {
				MessageBeep(MB_ICONEXCLAMATION);
				MessageBox(hWnd, "I am currently printing a graph; wait until I am done spooling, then try to exit again.",
							  "Exit", MB_OK | MB_ICONEXCLAMATION);
			 }
			 else
			 {
				SavePrefs(&gp, hWnd);
				DestroyWindow(hWnd);
				PostQuitMessage(0);
			 }
			 break;
		  }

		  case CM_GRAPHBOTH: case CM_GRAPHOFF: case CM_GRAPHDOTS:
		  {
			 ChangeGraphMenu(gp.Connect, LOWORD(wParam), NULL);
			 InvalidateRect(hWnd, NULL, TRUE);
			 gp.Connect = LOWORD(wParam);
			 GraphUpdated = TRUE;
			 break;
		  }

		  case CM_GRAPHSONAR:
		  {
			 ChangeMenuFlag(CM_GRAPHSONAR, &gp.Sonar, hWnd);
			 GraphUpdated = TRUE;
			 break;
		  }

		  case CM_GRAPHGRID:
		  {
			 ChangeMenuFlag(CM_GRAPHGRID, &gp.Grid, hWnd);
			 GraphUpdated = TRUE;
			 break;
		  }

		  case CM_GRAPHPRINT:
		  {
			 if (!ThreadRunning)
			 {
				DWORD ThreadID;
				if (!hThread)
				{
				  CloseHandle(hThread);
				  hThread = NULL;
				}

				// Bring up the "Printing..." Dialog
				hPrintDlg = CreateDialogParam(ghInstance, "PRINT_DIALOG",
														HWND_DESKTOP, (DLGPROC) PrintDlgProc,
														(LPARAM) &gp);
				SetActiveWindow(hPrintDlg);

				// Run the print thread
				hThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) PrintGraph,
											  &gp, 0, &ThreadID);
			 }
			 else
			 {
				MessageBeep(MB_ICONEXCLAMATION);
				MessageBox(hWnd, "I am already printing a graph; I cannot print more than one at a time.",
							  "Print Graph", MB_OK | MB_ICONEXCLAMATION);
			 }
			 break;
		  }

		  case CM_GRAPHPAGE_SETUP:
		  {
			 pds.hwndOwner = hWnd;
			 pds.Flags = PD_PRINTSETUP;
			 PrintDlg(&pds);
			 break;
		  }

		  case CM_GRAPHPROPERTIES:
		  {
			 DialogBox(ghInstance, "PROPERTIES", hWnd, (DLGPROC)
						  PropertiesDlgProc);
			 InvalidateRect(hWnd, NULL, TRUE);
			 break;
		  }
		  case CM_GRAPHABOUT:
		  {
			 DialogBox(ghInstance, "ABOUT", hWnd, (DLGPROC)
						  AboutDlgProc);
			 break;
		  }
		}
		return 0;
	 }
	 case WM_CLOSE :
	 {
		if (ThreadRunning)
		{
		  MessageBeep(MB_ICONEXCLAMATION);
		  MessageBox(hWnd, "I am currently printing a graph; wait until I am done spooling, then try to exit again.",
						 "Exit", MB_OK | MB_ICONEXCLAMATION);
		}
		else
		{
		  SavePrefs(&gp, hWnd);
		  DestroyWindow(hWnd);
		  PostQuitMessage(0);
		}
		return 0;
	 }
	 case WM_ERASEBKGND :
	 {
		return TRUE;
	 }
	 case WM_PAINT :
	 {
		// Windows wants us to update our window, so let's do that...
		HDC hDC, memDC;
		RECT ClientRect, UpdateRect;
		HBITMAP hOldBitmap;

		// Begin the painting and gather client dimensions
		GetClientRect(hWnd, &ClientRect);
		GetUpdateRect(hWnd, &UpdateRect, FALSE);
		hDC = GetDC(hWnd);

		// Create WinG DC
		if ((memDC = CreateCompatibleDC(hDC)) == 0) { ReleaseDC(hWnd, hDC); ValidateRect(hWnd, NULL); MessageBeep(MB_ICONSTOP); return 0; }

		// If the graph is not the same as the previous process, recalculate it
		if ((hGraph == 0) || (!EqualRect(&ClientRect, &GraphRect)) ||
			(GraphUpdated))
		{
		  int BitsPerPixel;

		  // Get important information about the DC
		  BitsPerPixel = GetDeviceCaps(hDC, BITSPIXEL);

		  // Delete previous bitmap, if any
		  if (hGraph != 0)
			 DeleteObject(hGraph);

		  // Create a bitmap for the graph
		  hGraph = CreateBitmap(ClientRect.right, ClientRect.bottom, 1, BitsPerPixel, NULL);
		  CopyRect(&GraphRect, &ClientRect);

		  // Draw the graph with labels
		  hOldBitmap = SelectObject(memDC, hGraph);
		  DrawGraphWithLabels(memDC, ClientRect, &gp);
		  GraphUpdated = FALSE;
		}
		else
		  hOldBitmap = SelectObject(memDC, hGraph);

		// Paste the memory device context to the screen
		BitBlt(hDC, UpdateRect.left, UpdateRect.top, UpdateRect.right,
				 UpdateRect.bottom, memDC, UpdateRect.left, UpdateRect.top, SRCCOPY);

		// Release Device Context and other objects
		SelectObject(memDC, hOldBitmap);
		ReleaseDC(hWnd, hDC);
		DeleteDC(memDC);
		ValidateRect(hWnd, NULL);
		return 0;
	 }
  }
  return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

void ChangeGraphMenu(int OldMenu, int NewMenu, LPSTR Name)
{
  if (OldMenu == NewMenu)
	 CheckMenuItem(hMenu, NewMenu, MF_CHECKED);
  else
  {
	 CheckMenuItem(hMenu, OldMenu, MF_UNCHECKED);
	 CheckMenuItem(hMenu, NewMenu, MF_CHECKED);
  }
  if (NewMenu == CM_GRAPHSINE)
	 lstrcpy(Name, "Sine");
  if (NewMenu == CM_GRAPHCOSINE)
	 lstrcpy(Name, "Cosine");
  if (NewMenu == CM_GRAPHTANGENT)
	 lstrcpy(Name, "Tangent");
  if (NewMenu == CM_GRAPHCOTANGENT)
	 lstrcpy(Name, "Cotangent");
  if (NewMenu == CM_GRAPHSECANT)
	 lstrcpy(Name, "Secant");
  if (NewMenu == CM_GRAPHCOSECANT)
	 lstrcpy(Name, "Cosecant");
}


//  Grid <--> Graph Conversion Functions:

//  These 4 functions are based on this equation

//  pt - min     cpt
//  ---------    ---
//  max - min    cs

// pt = Graph Point     min = Axis Minimum      max = Axis Maximum
// cpt = Client Point   cw = ClientSize
int GraphXToClient(double GraphPoint, int ClientWidth, GridProperties *g)
{
  return (int) StrangeRound(((GraphPoint - g->Modifier[M_XMIN]) * ClientWidth) / (g->Modifier[M_XMAX] - g->Modifier[M_XMIN]), 3);
}

int GraphYToClient(double GraphPoint, int ClientHeight, GridProperties *g)
{
  return (int) StrangeRound(((g->Modifier[M_YMAX] - GraphPoint) * ClientHeight) / (g->Modifier[M_YMAX] - g->Modifier[M_YMIN]), 3);
}

double ClientToGraphX(int GraphPoint, int ClientWidth, GridProperties *g)
{
  return ((GraphPoint * (g->Modifier[M_XMAX] - g->Modifier[M_XMIN])) / ClientWidth + g->Modifier[M_XMIN]);
}

double ClientToGraphY(int GraphPoint, int ClientHeight, GridProperties *g)
{
  return (0 - ((GraphPoint * (g->Modifier[M_YMAX] - g->Modifier[M_YMIN])) / ClientHeight - g->Modifier[M_YMIN]));
}

int StrangeRound(double x, int p)
{
  int lcv;

  // This function does a strange round; when given a number and a number
  // of decimal places, StrangeRound starts from the specified decimal spot
  // and rounds until it derives an integer.  In other words, it rounds to
  // the specified number of decimal places and loops until an integer is
  // derived, rounding to the previous decimal place through each pass of
  // the loop until a decimal point is reached.  For example, the number
  // 4.44444444521, when rounded with 8 decimal places, would produce 5.
  for(lcv = p; lcv >= 1; lcv--)
  {
	 if ((((x * pow(10, lcv - 1) - floor(x * pow(10, lcv - 1))) * 10) >= 5) &&
		 (((x * pow(10, lcv - 1) - floor(x * pow(10, lcv - 1)))) < 9))
		x = x + pow(10, 0 - (lcv - 1));
  }
  return (int) x;
}

int PrintGraph(GridProperties *g)
{
  GridProperties PrintGrid;

  // Initialize Variables
  ThreadRunning = TRUE;
  pds.Flags = PD_RETURNDC | PD_NOSELECTION;
  CopyMemory(&PrintGrid, g, sizeof(GridProperties));
  pds.hwndOwner = hPrintDlg;

  if (PrintDlg(&pds))
  {
	 RECT PrintRect;
	 DOCINFO di;
	 char DocumentName[40];

	 // Set the document name
	 ShowWindow(hPrintDlg, SW_MINIMIZE);
	 ZeroMemory(DocumentName, sizeof(char) * 40);
	 lstrcpy(DocumentName, PrintGrid.Name);
	 lstrcat(DocumentName, " - Trigonometric Grapher");
	 di.lpszDocName = DocumentName;
	 di.lpszOutput = NULL;

	 // Suspend the thread for two seconds (just in case the user changes mind)
	 Sleep(2000);

	 // Start printing
	 StartDoc(pds.hDC, &di);
	 StartPage(pds.hDC);

	 // Initialize dimensions
	 PrintRect.left = 0;
	 PrintRect.top = 0;
	 PrintRect.right = GetDeviceCaps(pds.hDC, HORZRES);
	 PrintRect.bottom = GetDeviceCaps(pds.hDC, VERTRES);

	 // Print
	 DrawGraphWithLabels(pds.hDC, PrintRect, &PrintGrid);

	 // End printing
	 SendMessage(hPrintDlg, WM_COMMAND, MAKEWPARAM(100, NULL), (LPARAM) NULL);
	 EndPage(pds.hDC);
	 if (Canceled)
		AbortDoc(pds.hDC);
	 else
		EndDoc(pds.hDC);
  }
  else
	 SendMessage(hPrintDlg, WM_COMMAND, MAKEWPARAM(100, NULL), (LPARAM) NULL);

  // End thread
  ThreadRunning = FALSE;
  ExitThread(0);
  return 0;
}

void DrawGraphWithLabels(HDC hDC, RECT ClientRect, GridProperties *g)
{
  DrawGraph(hDC, ClientRect, g);
}

void DrawGraph(HDC hDC, RECT ClientRect, GridProperties *g)
{
  COLORREF OldBkColor;
  POINT Orgin;
  HPEN hPen, hOldPen;
  HBRUSH hBrush, hOldBrush;
  SIZE oldSize;
  double gx, gy;
  int lcv;
  double dSonarWidth, dTickSize;
  int SonarWidth, TickSize;

  // Initialize Variables
  ClientRect.right = ClientRect.right - 1;
  ClientRect.bottom = ClientRect.bottom - 1;

  // Paint the Background
  hBrush = CreateSolidBrush(gp.Color[BACKGROUND]);
  hPen = CreatePen(PS_SOLID, 1, gp.Color[BACKGROUND]);
  hOldBrush = SelectObject(hDC, hBrush);
  hOldPen = SelectObject(hDC, hPen);
  Rectangle(hDC, ClientRect.left, ClientRect.top, ClientRect.right + 1,
				ClientRect.bottom + 1);
  SelectObject(hDC, hOldBrush);
  SelectObject(hDC, hOldPen);
  DeleteObject(hPen);
  DeleteObject(hBrush);

  // Check variables to prevent a crash
  if ((g->Modifier[M_XMIN] == g->Modifier[M_XMAX]) ||
		(g->Modifier[M_YMIN] == g->Modifier[M_YMAX]) ||
		(g->Modifier[M_XTICK] < 0) ||
		(g->Modifier[M_YTICK] < 0) ||
		(g->Modifier[M_XMIN] > g->Modifier[M_XMAX]) ||
		(g->Modifier[M_YMIN] > g->Modifier[M_YMAX]))
  {
	 char ErrorStr[] = "Error in Graph.  Check the Grid Properties by pressing [Alt]+[Enter].";
	 TextOut(hDC, 0, 0, ErrorStr, lstrlen(ErrorStr) * sizeof(char));
	 return;
  }

  // Initialize Variables
  Orgin.x = GraphXToClient(0, ClientRect.right, g);
  Orgin.y = GraphYToClient(0, ClientRect.bottom, g);
  dSonarWidth = ((ClientRect.right * ClientRect.bottom) * (g->Modifier[M_SONARWIDTH] / 100));
  dTickSize = ((ClientRect.right * ClientRect.bottom) * (g->Modifier[M_TICKSIZE] / 100));
  if (dSonarWidth <= 0)
	 SonarWidth = 0;
  else
	 SonarWidth = (int) sqrt(dSonarWidth);
  if (dTickSize <= 0)
	 TickSize = 0;
  else
	 TickSize = (int) sqrt(dTickSize);
  NextPlot = 0;

  // Set viewport and background color
  OldBkColor = SetBkColor(hDC, g->Color[BACKGROUND]);
  SetViewportExtEx(hDC, ClientRect.right, ClientRect.bottom, &oldSize);

  // Draw X and Y Axis
  hPen = CreatePen(PS_SOLID, 1, g->Color[AXIS]);
  hOldPen = SelectObject(hDC, hPen);
  MoveToEx(hDC, Orgin.x, ClientRect.left, NULL);
  LineTo(hDC, Orgin.x, ClientRect.bottom);
  MoveToEx(hDC, ClientRect.top, Orgin.y, NULL);
  LineTo(hDC, ClientRect.right, Orgin.y);
  SelectObject(hDC, hOldPen);
  DeleteObject(hPen);

  if (g->Grid)
  {
	 // Draw Grid Lines
	 hPen = CreatePen(PS_DOT, 1, g->Color[GRID]);
	 hOldPen = SelectObject(hDC, hPen);

	 // Vertical
	 lcv = 1;
	 while (lcv * g->Modifier[M_XTICK] <= g->Modifier[M_XMAX])
	 {
		gx = GraphXToClient(lcv * g->Modifier[M_XTICK], ClientRect.right, g);
		MoveToEx(hDC, gx, 0, NULL);
		LineTo(hDC, gx, ClientRect.bottom);
		lcv++;
	 }
	 lcv = -1;
	 while (lcv * g->Modifier[M_XTICK] >= g->Modifier[M_XMIN])
	 {
		gx = GraphXToClient(lcv * g->Modifier[M_XTICK], ClientRect.right, g);
		MoveToEx(hDC, gx, 0, NULL);
		LineTo(hDC, gx, ClientRect.bottom);
		lcv--;
	 }

	 // Horizontal
	 lcv = 1;
	 while (lcv * g->Modifier[M_YTICK] <= g->Modifier[M_YMAX])
	 {
		gy = GraphYToClient(lcv * g->Modifier[M_YTICK], ClientRect.bottom, g);
		MoveToEx(hDC, 0, gy, NULL);
		LineTo(hDC, ClientRect.right, gy);
		lcv++;
	 }
	 lcv = -1;
	 while (lcv * g->Modifier[M_YTICK] >= g->Modifier[M_YMIN])
	 {
		gy = GraphYToClient(lcv * g->Modifier[M_YTICK], ClientRect.bottom, g);
		MoveToEx(hDC, 0, gy, NULL);
		LineTo(hDC, ClientRect.right, gy);
		lcv--;
	 }

	 SelectObject(hDC, hOldPen);
	 DeleteObject(hPen);
  }

  // Draw tick marks
  hPen = CreatePen(PS_SOLID, 1, g->Color[TICK]);
  hOldPen = SelectObject(hDC, hPen);

  // Vertical
  lcv = 1;
  while (lcv * g->Modifier[M_XTICK] <= g->Modifier[M_XMAX])
  {
	 gx = GraphXToClient(lcv * g->Modifier[M_XTICK], ClientRect.right, g);
	 MoveToEx(hDC, gx, Orgin.y - MulDiv(1, TickSize, 2), NULL);
	 LineTo(hDC, gx, Orgin.y + MulDiv(1, TickSize, 2) + 1);
	 lcv++;
  }
  lcv = -1;
  while (lcv * g->Modifier[M_XTICK] >= g->Modifier[M_XMIN])
  {
	 gx = GraphXToClient(lcv * g->Modifier[M_XTICK], ClientRect.right, g);
	 MoveToEx(hDC, gx, Orgin.y - MulDiv(1, TickSize, 2), NULL);
	 LineTo(hDC, gx, Orgin.y + MulDiv(1, TickSize, 2) + 1);
	 lcv--;
  }

  // Horizontal
  lcv = 1;
  while (lcv * g->Modifier[M_YTICK] <= g->Modifier[M_YMAX])
  {
	 gy = GraphYToClient(lcv * g->Modifier[M_YTICK], ClientRect.bottom, g);
	 MoveToEx(hDC, Orgin.x - MulDiv(1, TickSize, 2), gy, NULL);
	 LineTo(hDC, Orgin.x + MulDiv(1, TickSize, 2) + 1, gy);
	 lcv++;
  }
  lcv = -1;
  while (lcv * g->Modifier[M_YTICK] >= g->Modifier[M_YMIN])
  {
	 gy = GraphYToClient(lcv * g->Modifier[M_YTICK], ClientRect.bottom, g);
	 MoveToEx(hDC, Orgin.x - MulDiv(1, TickSize, 2), gy, NULL);
	 LineTo(hDC, Orgin.x + MulDiv(1, TickSize, 2) + 1, gy);
	 lcv--;
  }
  SelectObject(hDC, hOldPen);
  DeleteObject(hPen);

  // Draw Graph
  if (g->Connect == CM_GRAPHDOTS)
  {
	 // Graph Dots
	 for(lcv = 0; lcv <= ClientRect.right; lcv++)
	 {
		gx = ClientToGraphX(lcv, ClientRect.right, g);

		if (GetTrigValue(gx, &gy, g->Graph, g->Modifier))
		  SetPixelV(hDC, lcv, GraphYToClient(gy, ClientRect.bottom, g), g->Color[CONNECT]);
	 }
  }
  if (g->Connect == CM_GRAPHBOTH)
  {
	 // Graph Dots and Lines
	 BOOL Graphed = FALSE;
	 BOOL XAsyn = !GetTrigValue(0, &gy, g->Graph, g->Modifier);
	 double Step = g->Modifier[M_XTICK];
	 double PrevStep;
	 double px = 0;
	 int Step1 = GraphXToClient(g->Modifier[M_XTICK], ClientRect.right, g), Step2 = 0;
	 int cx;
	 hPen = CreatePen(PS_SOLID, 1, g->Color[CONNECT]);
	 hOldPen = SelectObject(hDC, hPen);

	 while (Step1 != Step2)
	 {
		Step2 = Step1;
		PrevStep = Step;
		Step = Step / 2;
		Step1 = GraphXToClient(Step, ClientRect.right, g);
	 }
	 Step = PrevStep;
	 gy = 0;
	 if (!XAsyn)
		lcv = 0;
	 else
		lcv = 1;
	 while (lcv * Step <= g->Modifier[M_XMAX])
	 {
		gx = lcv * Step;
		cx = GraphXToClient(gx, ClientRect.right, g);
		if (!Graphed)
		{
		  if (GetTrigValue(gx, &gy, g->Graph, g->Modifier))
		  {
			 MoveToEx(hDC, cx, GraphYToClient(gy, ClientRect.bottom, g), NULL);
			 Graphed = TRUE;
		  }
		}
		else
		{
		  if (GetTrigValue(gx, &gy, g->Graph, g->Modifier))
		  {
			 if (cx != px)
				LineTo(hDC, cx, GraphYToClient(gy, ClientRect.bottom, g));
			 px = cx;
		  }
		  else
		  {
			 DrawAsymptote(hDC, gx, ClientRect.right, ClientRect.bottom, Orgin.x, g);
			 Graphed = FALSE;
		  }
		}
		lcv++;
	 }
	 Graphed = FALSE;
	 if (!XAsyn)
		lcv = 0;
	 else
		lcv = -1;
	 while (lcv * Step >= g->Modifier[M_XMIN])
	 {
		gx = lcv * Step;
		cx = GraphXToClient(gx, ClientRect.right, g);
		if (!Graphed)
		{
		  if (GetTrigValue(gx, &gy, g->Graph, g->Modifier))
		  {
			 MoveToEx(hDC, cx, GraphYToClient(gy, ClientRect.bottom, g), NULL);
			 Graphed = TRUE;
		  }
		}
		else
		{
		  if (GetTrigValue(gx, &gy, g->Graph, g->Modifier))
		  {
			 if (cx != px)
				LineTo(hDC, cx, GraphYToClient(gy, ClientRect.bottom, g));
			 px = cx;
		  }
		  else
		  {
			 DrawAsymptote(hDC, gx, ClientRect.right, ClientRect.bottom, Orgin.x, g);
			 Graphed = FALSE;
		  }
		}
		lcv--;
	 }
	 if (XAsyn)
		DrawAsymptote(hDC, 0, ClientRect.right, ClientRect.bottom, Orgin.x, g);
	 SelectObject(hDC, hOldPen);
	 DeleteObject(hPen);
  }

  // Plot Points
  hPen = CreatePen(PS_SOLID, 1, g->Color[SONAR]);
  hOldPen = SelectObject(hDC, hPen);
  lcv = 0;
  while (lcv * g->Modifier[M_XTICK] <= g->Modifier[M_XMAX])
  {
	 gx = lcv * g->Modifier[M_XTICK];

	 if (GetTrigValue(gx, &gy, g->Graph, g->Modifier))
	 {
		SetPixelV(hDC, GraphXToClient(gx, ClientRect.right, g), GraphYToClient(gy, ClientRect.bottom, g), g->Color[DOT]);
		if (g->Sonar)
		{
		  MoveToEx(hDC, GraphXToClient(gx, ClientRect.right, g) - MulDiv(1, SonarWidth, 2), GraphYToClient(gy, ClientRect.bottom, g) - MulDiv(1, SonarWidth, 2), NULL);
		  LineTo(hDC, GraphXToClient(gx, ClientRect.right, g) + MulDiv(1, SonarWidth, 2), GraphYToClient(gy, ClientRect.bottom, g) - MulDiv(1, SonarWidth, 2));
		  LineTo(hDC, GraphXToClient(gx, ClientRect.right, g) + MulDiv(1, SonarWidth, 2), GraphYToClient(gy, ClientRect.bottom, g) + MulDiv(1, SonarWidth, 2));
		  LineTo(hDC, GraphXToClient(gx, ClientRect.right, g) - MulDiv(1, SonarWidth, 2), GraphYToClient(gy, ClientRect.bottom, g) + MulDiv(1, SonarWidth, 2));
		  LineTo(hDC, GraphXToClient(gx, ClientRect.right, g) - MulDiv(1, SonarWidth, 2), GraphYToClient(gy, ClientRect.bottom, g) - MulDiv(1, SonarWidth, 2));
		}
	 }
	 lcv++;
  }
  lcv = -1;
  while (lcv * g->Modifier[M_XTICK] >= g->Modifier[M_XMIN])
  {
	 gx = lcv * g->Modifier[M_XTICK];

	 if (GetTrigValue(gx, &gy, g->Graph, g->Modifier))
	 {
		SetPixelV(hDC, GraphXToClient(gx, ClientRect.right, g), GraphYToClient(gy, ClientRect.bottom, g), g->Color[DOT]);
		if (g->Sonar)
		{
		  MoveToEx(hDC, GraphXToClient(gx, ClientRect.right, g) - MulDiv(1, SonarWidth, 2), GraphYToClient(gy, ClientRect.bottom, g) - MulDiv(1, SonarWidth, 2), NULL);
		  LineTo(hDC, GraphXToClient(gx, ClientRect.right, g) + MulDiv(1, SonarWidth, 2), GraphYToClient(gy, ClientRect.bottom, g) - MulDiv(1, SonarWidth, 2));
		  LineTo(hDC, GraphXToClient(gx, ClientRect.right, g) + MulDiv(1, SonarWidth, 2), GraphYToClient(gy, ClientRect.bottom, g) + MulDiv(1, SonarWidth, 2));
		  LineTo(hDC, GraphXToClient(gx, ClientRect.right, g) - MulDiv(1, SonarWidth, 2), GraphYToClient(gy, ClientRect.bottom, g) + MulDiv(1, SonarWidth, 2));
		  LineTo(hDC, GraphXToClient(gx, ClientRect.right, g) - MulDiv(1, SonarWidth, 2), GraphYToClient(gy, ClientRect.bottom, g) - MulDiv(1, SonarWidth, 2));
		}
	 }
	 else
		DrawAsymptote(hDC, gx, ClientRect.right, ClientRect.bottom, Orgin.x, g);
	 lcv--;
  }
  SelectObject(hDC, hOldPen);
  DeleteObject(hPen);

  // Set viewport to original settings
  SetViewportExtEx(hDC, oldSize.cx, oldSize.cy, NULL);
  SetBkColor(hDC, OldBkColor);
}

double round(double x, int p)
{
  // Round x to the nearest given decimal place (p)
  // The reason I had to program this results from the lack of
  // a rounding function in C++ (I wonder why they would leave it
  // out?).
  x = x * pow(10, p);
  if (x - floor(x) >= .5)
	 x = x + 1;
  return floor(x) / pow(10, p);
}

BOOL GetTrigValue(double x, double *y, int Method, double *Modifier)
{
  double sx = sin(Modifier[M_B_MODIFIER] * x + Modifier[M_C_MODIFIER]);
  double cx = cos(Modifier[M_B_MODIFIER] * x + Modifier[M_C_MODIFIER]);

  switch (Method)
  {
	 case CM_GRAPHSINE      : { *y = Modifier[M_A_MODIFIER] * sx + Modifier[M_D_MODIFIER]; break; }
	 case CM_GRAPHCOSINE    : { *y = Modifier[M_A_MODIFIER] * cx + Modifier[M_D_MODIFIER]; break; }
	 case CM_GRAPHTANGENT   : { if (round(cx, 4) == 0) return FALSE; *y = Modifier[M_A_MODIFIER] * (sx / cx) + Modifier[M_D_MODIFIER]; break; }
	 case CM_GRAPHCOTANGENT : { if (round(sx, 4) == 0) return FALSE; *y = Modifier[M_A_MODIFIER] * (cx / sx) + Modifier[M_D_MODIFIER]; break; }
	 case CM_GRAPHSECANT    : { if (round(cx, 4) == 0) return FALSE; *y = Modifier[M_A_MODIFIER] * (1 / cx) + Modifier[M_D_MODIFIER]; break; }
	 case CM_GRAPHCOSECANT  : { if (round(sx, 4) == 0) return FALSE; *y = Modifier[M_A_MODIFIER] * (1 / sx) + Modifier[M_D_MODIFIER]; break; }
  }

  return TRUE;
}

void DrawAsymptote(HDC hDC, double x, int w, int h, int XOrgin, GridProperties *g)
{
  HPEN hPen, hOldPen;
  POINT OldPoint;
  COLORREF bkColor;

  hPen = CreatePen(PS_DASH, 1, g->Color[ASYMPTOTE]);
  hOldPen = SelectObject(hDC, hPen);
  if (GraphXToClient(x, w, g) == XOrgin)
	 bkColor = SetBkColor(hDC, g->Color[AXIS]);

  MoveToEx(hDC, GraphXToClient(x, w, g), 0, &OldPoint);
  LineTo(hDC, GraphXToClient(x, w, g), h);
  MoveToEx(hDC, OldPoint.x, OldPoint.y, NULL);

  if (GraphXToClient(x, w, g) == XOrgin)
	 SetBkColor(hDC, bkColor);
  SelectObject(hDC, hOldPen);
  DeleteObject(hPen);
}

#pragma argsused
BOOL CALLBACK PrintDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
	 case WM_INITDIALOG :
	 {
		GridProperties *g = (GridProperties FAR *) lParam;
		Canceled = FALSE;

		// Set Name
		SetDlgItemText(hDlg, ID_GRAPH, g->Name);

		// Set Connection Method
		if (g->Connect == CM_GRAPHDOTS)
		  SetDlgItemText(hDlg, ID_CONNECT, "Unconnected Dots");
		if (g->Connect == CM_GRAPHBOTH)
		  SetDlgItemText(hDlg, ID_CONNECT, "Connected Dots");
		if (g->Connect == CM_GRAPHOFF)
		  SetDlgItemText(hDlg, ID_CONNECT, "No Dot Connection");

		// Set Grid Lines
		if (g->Grid)
		  SetDlgItemText(hDlg, ID_GRID, "On");
		else
		  SetDlgItemText(hDlg, ID_GRID, "Off");

		// Set Sonar Status
		if (g->Sonar)
		  SetDlgItemText(hDlg, ID_SONAR, "On");
		else
		  SetDlgItemText(hDlg, ID_SONAR, "Off");
		return TRUE;
	 }
	 case WM_COMMAND :
	 {
		switch (LOWORD(wParam))
		{
		  case IDCANCEL : { Canceled = TRUE; break; }
		  case 100      : { DestroyWindow(hDlg); break; }
		}
		return 0;
	 }
	 case WM_CLOSE :
	 {
		Canceled = TRUE;
		return 0;
	 }
  }
  return FALSE;
}

#pragma argsused
BOOL CALLBACK AboutDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
	 case WM_INITDIALOG :
	 {
		return TRUE;
	 }
	 case WM_COMMAND :
	 {
		switch (LOWORD(wParam))
		{
		  case IDCANCEL :
		  case IDOK :
		  {
			 EndDialog(hDlg, 0);
			 break;
		  }
		}
		return 0;
	 }
  }
  return FALSE;
}

void SetGraphWindowText(HWND hWnd, LPSTR Text)
{
  char WindowText[40];

  ZeroMemory(WindowText, sizeof(char) * 40);
  lstrcpy(WindowText, Text);
  lstrcat(WindowText, " - Trigonometric Grapher");

  SetWindowText(hWnd, WindowText);
}

void SavePrefs(GridProperties *g, HWND hWnd)
{
  HKEY hKey;
  DWORD Disposition;
  RECT WindowRect;
  int lcv;
  char Name[NAMELEN];

  // Restore twice in case if the user maximized AND minimized
  ShowWindow(hWnd, SW_RESTORE);
  ShowWindow(hWnd, SW_RESTORE);
  GetWindowRect(hWnd, &WindowRect);
  WindowRect.right = WindowRect.right - WindowRect.left;
  WindowRect.bottom = WindowRect.bottom - WindowRect.top;

  // Save our preferences into the registry
  RegCreateKeyEx(HKEY_CURRENT_USER, "Software\\Steven Lawrance\\Trigonometric Grapher",
					  0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
					  &hKey, &Disposition);
  RegSetValueEx(hKey, "Window X Placement", 0, REG_DWORD, (LPBYTE) &WindowRect.left,
				  (DWORD) sizeof(int));
  RegSetValueEx(hKey, "Window Y Placement", 0, REG_DWORD, (LPBYTE) &WindowRect.top,
				  (DWORD) sizeof(int));
  RegSetValueEx(hKey, "Window Width", 0, REG_DWORD, (LPBYTE) &WindowRect.right,
				  (DWORD) sizeof(int));
  RegSetValueEx(hKey, "Window Height", 0, REG_DWORD, (LPBYTE) &WindowRect.bottom,
				  (DWORD) sizeof(int));
  RegSetValueEx(hKey, "Grid", 0, REG_BINARY, (LPBYTE) &(g->Grid), (DWORD) sizeof(BOOL));
  RegSetValueEx(hKey, "Sonar", 0, REG_BINARY, (LPBYTE) &(g->Sonar), (DWORD) sizeof(BOOL));
  RegSetValueEx(hKey, "Connect Method", 0, REG_DWORD, (LPBYTE) &(g->Connect),
				  (DWORD) sizeof(int));
  RegSetValueEx(hKey, "Graph Method", 0, REG_DWORD, (LPBYTE) &(g->Graph),
				  (DWORD) sizeof(int));

  // Save colors
  for (lcv = ASYMPTOTE; lcv <= TICK; lcv++)
  {
	 LoadString(ghInstance, lcv, Name, sizeof(char) * NAMELEN);
	 RegSetValueEx(hKey, Name, 0, REG_DWORD, (LPBYTE) &(g->Color[lcv]),
						(DWORD) sizeof(int));
  }

  // Save modifiers
  for (lcv = 0; lcv <= NUM_MODIFIERS - 1; lcv++)
  {
	 LoadString(ghInstance, lcv + 200, Name, sizeof(char) * NAMELEN);
	 RegSetValueEx(hKey, Name, 0, REG_SZ, (LPBYTE) &(g->ModifierStr[lcv]),
						(DWORD) lstrlen(g->ModifierStr[lcv]));
  }

  // Close the key
  RegCloseKey(hKey);
}

void LoadPrefs(GridProperties *g, HWND hWnd)
{
  HKEY hKey;
  RECT WindowRect;
  int lcv;
  char Name[NAMELEN];

  // Set defaults (in case if some values are missing from the registry)
  g->Grid = TRUE;
  g->Connect = CM_GRAPHBOTH;
  g->Sonar = FALSE;
  g->Graph = CM_GRAPHSINE;

  // Colors
  g->Color[ASYMPTOTE] = RGB(255, 0, 0);
  g->Color[AXIS] = RGB(0, 0, 0);
  g->Color[BACKGROUND] = RGB(255, 255, 255);
  g->Color[GRID] = RGB(0, 128, 128);
  g->Color[DOT] = RGB(255, 0, 0);
  g->Color[CONNECT] = RGB(0, 0, 255);
  g->Color[SONAR] = RGB(255, 0, 0);
  g->Color[TICK] = RGB(0, 0, 255);

  // Window Placement
  WindowRect.left = 0;
  WindowRect.top = 0;
  WindowRect.right = 320;
  WindowRect.bottom = 200;

  // Modifiers
  for (lcv = 0; lcv <= NUM_MODIFIERS - 1; lcv++)
  {
	 LoadString(ghInstance, lcv + 300, Name, sizeof(char) * NAMELEN);
	 lstrcpy(g->ModifierStr[lcv], Name);
  }
  CalculateModifiers(g);

  if (RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\Steven Lawrance\\Trigonometric Grapher",
					0, KEY_READ, &hKey) != ERROR_SUCCESS)
  {
	 // Apply the default settings
	 ChangeGraphMenu(g->Graph, g->Graph, g->Name);
	 SetGraphWindowText(hWnd, g->Name);
	 ChangeGraphMenu(g->Connect, g->Connect, NULL);
	 CheckMenuItem(hMenu, CM_GRAPHSONAR, MF_UNCHECKED);
	 CheckMenuItem(hMenu, CM_GRAPHGRID, MF_CHECKED);
  }
  else
  {
	 DWORD Type;
	 DWORD Size;

	 // Load our preferences from the registry
	 Size = (DWORD) sizeof(int);
	 RegQueryValueEx(hKey, "Window X Placement", NULL, &Type, (LPBYTE) &WindowRect.left, &Size);
	 Size = (DWORD) sizeof(int);
	 RegQueryValueEx(hKey, "Window Y Placement", NULL, &Type, (LPBYTE) &WindowRect.top, &Size);
	 Size = (DWORD) sizeof(int);
	 RegQueryValueEx(hKey, "Window Width", NULL, &Type, (LPBYTE) &WindowRect.right, &Size);
	 Size = (DWORD) sizeof(int);
	 RegQueryValueEx(hKey, "Window Height", NULL, &Type, (LPBYTE) &WindowRect.bottom, &Size);
	 Size = (DWORD) sizeof(BOOL);
	 RegQueryValueEx(hKey, "Grid", NULL, &Type, (LPBYTE) &(g->Grid), &Size);
	 Size = (DWORD) sizeof(BOOL);
	 RegQueryValueEx(hKey, "Sonar", NULL, &Type, (LPBYTE) &(g->Sonar), &Size);
	 Size = (DWORD) sizeof(int);
	 RegQueryValueEx(hKey, "Connect Method", NULL, &Type, (LPBYTE) &(g->Connect), &Size);
	 Size = (DWORD) sizeof(int);
	 RegQueryValueEx(hKey, "Graph Method", NULL, &Type, (LPBYTE) &(g->Graph), &Size);

	 // Load the colors
	 for (lcv = ASYMPTOTE; lcv <= TICK; lcv++)
	 {
		LoadString(ghInstance, lcv, Name, sizeof(char) * NAMELEN);
		Size = (DWORD) sizeof(int);
		RegQueryValueEx(hKey, Name, NULL, &Type, (LPBYTE) &(g->Color[lcv]), &Size);
	 }

	 // Load the modifiers
	 for (lcv = 0; lcv <= NUM_MODIFIERS - 1; lcv++)
	 {
		LoadString(ghInstance, lcv + 200, Name, sizeof(char) * NAMELEN);
		Size = (DWORD) sizeof(char) * NAMELEN;
		RegQueryValueEx(hKey, Name, NULL, &Type, (LPBYTE) &(g->ModifierStr[lcv]), &Size);
	 }
	 CalculateModifiers(g);

	 // Close the key
	 RegCloseKey(hKey);

	 // Apply the settings
	 ChangeGraphMenu(g->Graph, g->Graph, g->Name);
	 SetGraphWindowText(hWnd, g->Name);
	 ChangeGraphMenu(g->Connect, g->Connect, NULL);
	 if (g->Sonar)
		CheckMenuItem(hMenu, CM_GRAPHSONAR, MF_CHECKED);
	 else
		CheckMenuItem(hMenu, CM_GRAPHSONAR, MF_UNCHECKED);
	 if (g->Grid)
		CheckMenuItem(hMenu, CM_GRAPHGRID, MF_CHECKED);
	 else
		CheckMenuItem(hMenu, CM_GRAPHGRID, MF_UNCHECKED);
	 MoveWindow(hWnd, WindowRect.left, WindowRect.top, WindowRect.right,
					  WindowRect.bottom, TRUE);
  }
  GraphUpdated = TRUE;
}

void SaveCustomColors(DWORD *g)
{
  HKEY hKey;
  DWORD Disposition;

  RegCreateKeyEx(HKEY_CURRENT_USER, "Software\\Steven Lawrance\\Trigonometric Grapher\\Custom Colors",
					  0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
					  &hKey, &Disposition);
  {
	 int lcv;
	 char RegValue[15];

	 // Save our preferences into the registry
	 for (lcv = 0; lcv <= 15; lcv++)
	 {
		ZeroMemory(RegValue, sizeof(char) * 15);
		wsprintf(RegValue, "Color Value %i", lcv);
		RegSetValueEx(hKey, RegValue, 0, REG_DWORD, (LPBYTE) &g[lcv],
				  (DWORD) sizeof(int));
	 }
	 RegCloseKey(hKey);
  }
}

void LoadCustomColors(DWORD *g)
{
  HKEY hKey;

  // Set Defaults
  g[0] = RGB(0, 0, 0);
  g[1] = RGB(0, 128, 128);
  g[2] = RGB(0, 0, 255);
  g[3] = RGB(0, 0, 255);
  g[4] = RGB(255, 0, 0);
  g[5] = RGB(255, 0, 0);
  g[6] = RGB(255, 0, 0);
  g[7] = RGB(255, 255, 255);
  g[8] = RGB(255, 255, 255);
  g[9] = RGB(255, 255, 255);
  g[10] = RGB(255, 255, 255);
  g[11] = RGB(255, 255, 255);
  g[12] = RGB(255, 255, 255);
  g[13] = RGB(255, 255, 255);
  g[14] = RGB(255, 255, 255);
  g[15] = RGB(255, 255, 255);

  if (RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\Steven Lawrance\\Trigonometric Grapher\\Custom Colors",
					0, KEY_READ, &hKey) == ERROR_SUCCESS)
  {
	 DWORD Type;
	 DWORD Size;
	 int lcv;
	 char RegValue[15];

	 // Load our preferences from the registry
	 for (lcv = 0; lcv <= 15; lcv++)
	 {
		ZeroMemory(RegValue, sizeof(char) * 15);
		wsprintf(RegValue, "Color Value %i", lcv);
		Size = (DWORD) sizeof(int);
		RegQueryValueEx(hKey, RegValue, NULL, &Type, (LPBYTE) &g[lcv], &Size);
	 }
	 RegCloseKey(hKey);
  }
}

GridProperties prop, backup;
BOOL Click;
char VarConst[50];
BOOL CursorOn;

#pragma argsused
BOOL CALLBACK PropertiesDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
	 case WM_INITDIALOG :
	 {
		int lcv;
		char Name[NAMELEN];

		Click = FALSE;
		CopyMemory(&prop, &gp, sizeof(GridProperties));
		CopyMemory(&backup, &gp, sizeof(GridProperties));

		for(lcv = 0; lcv <= NUM_MODIFIERS - 1; lcv++)
		{
		  SetDlgItemText(hDlg, lcv + ID_A_MODIFIER, prop.ModifierStr[lcv]);
        SendDlgItemMessage(hDlg, lcv + ID_A_MODIFIER, EM_LIMITTEXT, NAMELEN - 1, 0);
		}

		// Load the color names into the color list box
		for (lcv = ASYMPTOTE; lcv <= TICK; lcv++)
		{
		  LoadString(ghInstance, lcv, Name, sizeof(char) * NAMELEN);
		  SendDlgItemMessage(hDlg, ID_ELEMENT, CB_ADDSTRING, 0, (LPARAM) Name);
		}
		SendDlgItemMessage(hDlg, ID_ELEMENT, CB_SETCURSEL, 0, 0);
		return TRUE;
	 }
	 case WM_MOUSEMOVE :
	 {
		if (Click)
		{
		  POINT pt;
		  HWND hFocus;
		  pt.x = LOWORD(lParam);
		  pt.y = HIWORD(lParam);
		  if ((hFocus = ChildWindowFromPoint(hDlg, pt)) != NULL)
		  {
			 if (((hFocus == GetDlgItem(hDlg, ID_TICKSIZE))
				 || (hFocus == GetDlgItem(hDlg, ID_SONARWIDTH))
				 || (hFocus == GetDlgItem(hDlg, ID_A_MODIFIER))
				 || (hFocus == GetDlgItem(hDlg, ID_B_MODIFIER))
				 || (hFocus == GetDlgItem(hDlg, ID_C_MODIFIER))
				 || (hFocus == GetDlgItem(hDlg, ID_D_MODIFIER))
				 || (hFocus == GetDlgItem(hDlg, ID_XMIN))
				 || (hFocus == GetDlgItem(hDlg, ID_XMAX))
				 || (hFocus == GetDlgItem(hDlg, ID_XTICK))
				 || (hFocus == GetDlgItem(hDlg, ID_YMIN))
				 || (hFocus == GetDlgItem(hDlg, ID_YMAX))
				 || (hFocus == GetDlgItem(hDlg, ID_YTICK))))
			 {
				if (!CursorOn)
				{
				  SetCursor(LoadCursor(NULL, IDC_UPARROW));
				  CursorOn = TRUE;
				}
			 }
			 else
			 {
				if (CursorOn)
				{
				  SetCursor(LoadCursor(NULL, IDC_NO));
				  CursorOn = FALSE;
				}
			 }
		  }
		}
		return 0;
	 }
	 case WM_LBUTTONDOWN :
	 {
		if (Click)
		{
		  POINT pt;
		  HWND hFocus;
		  Click = FALSE;
		  ReleaseCapture();
		  pt.x = LOWORD(lParam);
		  pt.y = HIWORD(lParam);
		  if ((hFocus = ChildWindowFromPoint(hDlg, pt)) != NULL)
		  {
			 if ((hFocus == GetDlgItem(hDlg, ID_TICKSIZE))
				 || (hFocus == GetDlgItem(hDlg, ID_SONARWIDTH))
				 || (hFocus == GetDlgItem(hDlg, ID_A_MODIFIER))
				 || (hFocus == GetDlgItem(hDlg, ID_B_MODIFIER))
				 || (hFocus == GetDlgItem(hDlg, ID_C_MODIFIER))
				 || (hFocus == GetDlgItem(hDlg, ID_D_MODIFIER))
				 || (hFocus == GetDlgItem(hDlg, ID_XMIN))
				 || (hFocus == GetDlgItem(hDlg, ID_XMAX))
				 || (hFocus == GetDlgItem(hDlg, ID_XTICK))
				 || (hFocus == GetDlgItem(hDlg, ID_YMIN))
				 || (hFocus == GetDlgItem(hDlg, ID_YMAX))
				 || (hFocus == GetDlgItem(hDlg, ID_YTICK)))
			 {
				// Add to the previous edit control (if any) and return control
				char EditText[NAMELEN];
				ZeroMemory(EditText, sizeof(char) * NAMELEN);
				GetWindowText(hFocus, EditText, NAMELEN - lstrlen(EditText));
				lstrcat(EditText, VarConst);
				SetWindowText(hFocus, EditText);
				SetFocus(hFocus);
			 }
		  }
		}
		return 0;
	 }
	 case WM_MEASUREITEM :
	 {
		LPMEASUREITEMSTRUCT lpmis = (LPMEASUREITEMSTRUCT) lParam;
		RECT rect;

		// Inform Windows about the size of the little button
		GetClientRect(GetDlgItem(hDlg, ID_COLOR), &rect);
		lpmis->CtlType = ODT_BUTTON;
		lpmis->CtlID = ID_COLOR;
		lpmis->itemID = 0;
		lpmis->itemWidth = rect.right;
		lpmis->itemHeight = rect.bottom;
		lpmis->itemData = 0;
		return TRUE;
	 }
	 case WM_DRAWITEM :
	 {
		LPDRAWITEMSTRUCT lpdis = (LPDRAWITEMSTRUCT) lParam;
		RECT rect;

		GetClientRect(GetDlgItem(hDlg, ID_COLOR), &rect);
		if (lpdis->CtlID == ID_COLOR)
		{
		  if (lpdis->itemAction == ODA_DRAWENTIRE)
		  {
			 // Draw the selected color
			 HBRUSH hBrush, hOldBrush;
			 HPEN hPen, hOldPen;

			 // Create a brush and a pen.  Select them into the device context...
			 hBrush = CreateSolidBrush(prop.Color[SendDlgItemMessage(hDlg, ID_ELEMENT, CB_GETCURSEL, 0, 0)]);
			 hPen = CreatePen(PS_SOLID, 1, prop.Color[SendDlgItemMessage(hDlg, ID_ELEMENT, CB_GETCURSEL, 0, 0)]);
			 hOldBrush = SelectObject(lpdis->hDC, hBrush);
			 hOldPen = SelectObject(lpdis->hDC, hPen);

			 // Draw the rectangle
			 Rectangle(lpdis->hDC, rect.left, rect.top, rect.right, rect.bottom);

			 // Unselect the objects and delete the brush and the pen
			 SelectObject(lpdis->hDC, hOldBrush);
			 SelectObject(lpdis->hDC, hOldPen);
			 DeleteObject(hPen);
			 DeleteObject(hBrush);
		  }
		}
		return TRUE;
	 }

	 case WM_COMMAND :
	 {
		switch (LOWORD(wParam))
		{
		  case IDCANCEL :
		  {
			 // Copy the backup GridProperties into the main window (canceling
			 // any changes)
			 CopyMemory(&gp, &backup, sizeof(GridProperties));
			 GraphUpdated = TRUE;
			 EndDialog(hDlg, 0);
			 break;
		  }
		  case ID_CHANGECOLOR :
		  {
			 // The user wants to change the color; load the custom colors and
			 // display the common dialog box for the colors
			 CHOOSECOLOR cc;
			 DWORD CustomColors[16];
			 LoadCustomColors(CustomColors);
			 cc.lStructSize = sizeof(CHOOSECOLOR);
			 cc.hwndOwner = hDlg;
			 cc.hInstance = NULL;
			 cc.rgbResult = (DWORD) prop.Color[SendDlgItemMessage(hDlg, ID_ELEMENT, CB_GETCURSEL, 0, 0)];
			 cc.lpCustColors = CustomColors;
			 cc.Flags = CC_RGBINIT | CC_FULLOPEN;
			 cc.lCustData = (DWORD) NULL;
			 cc.lpfnHook = NULL;
			 cc.lpTemplateName = NULL;
			 if (ChooseColor(&cc))
			 {
				prop.Color[SendDlgItemMessage(hDlg, ID_ELEMENT, CB_GETCURSEL, 0, 0)] = cc.rgbResult;
				SaveCustomColors(CustomColors);
				InvalidateRect(GetDlgItem(hDlg, ID_COLOR), NULL, TRUE);
			 }
			 break;
		  }
		  case ID_ELEMENT :
		  {
			 if (HIWORD(wParam) == CBN_SELCHANGE)
				InvalidateRect(GetDlgItem(hDlg, ID_COLOR), NULL, TRUE);
			 break;
		  }
		  case ID_ADDVAR :
		  {
			 ZeroMemory(VarConst, sizeof(char) * 50);
			 lstrcpy(VarConst, "p");
			 Click = TRUE;
			 CursorOn = FALSE;
			 SetCursor(LoadCursor(NULL, IDC_NO));
			 SetCapture(hDlg);
			 break;
		  }
		  case IDOK :
		  case ID_APPLY :
		  {
			 // Apply the settings to the main window
			 int lcv;
			 for(lcv = 0; lcv <= NUM_MODIFIERS - 1; lcv++)
				GetDlgItemText(hDlg, lcv + ID_A_MODIFIER, prop.ModifierStr[lcv], NAMELEN);
			 CalculateModifiers(&prop);

			 // Check ranges.  If the ranges conflict, fix them after notifying user
			 if (prop.Modifier[M_XMIN] > prop.Modifier[M_XMAX])
			 {
				double temp;
				char Name[NAMELEN];
				MessageBeep(MB_ICONEXCLAMATION);
				if (MessageBox(hDlg, "Your X Minimum is more than your X Maximum!  Do you want to swap the values.", "Range Error", MB_YESNO | MB_ICONEXCLAMATION) == IDYES)
				{
				  temp = prop.Modifier[M_XMAX];
				  prop.Modifier[M_XMAX] = prop.Modifier[M_XMIN];
				  prop.Modifier[M_XMIN] = temp;
				  lstrcpy(Name, prop.ModifierStr[M_XMAX]);
				  lstrcpy(prop.ModifierStr[M_XMAX], prop.ModifierStr[M_XMIN]);
				  lstrcpy(prop.ModifierStr[M_XMIN], Name);
				  SetDlgItemText(hDlg, ID_XMIN, prop.ModifierStr[M_XMIN]);
				  SetDlgItemText(hDlg, ID_XMAX, prop.ModifierStr[M_XMAX]);
				}
				else
				  break;
			 }
			 if (prop.Modifier[M_YMIN] > prop.Modifier[M_YMAX])
			 {
				double temp;
				char Name[NAMELEN];
				MessageBeep(MB_ICONEXCLAMATION);
				if (MessageBox(hDlg, "Your Y Minimum is more than your Y Maximum!  Do you want to swap the values.", "Range Error", MB_YESNO | MB_ICONEXCLAMATION) == IDYES)
				{
				  temp = prop.Modifier[M_YMAX];
				  prop.Modifier[M_YMAX] = prop.Modifier[M_YMIN];
				  prop.Modifier[M_YMIN] = temp;
				  lstrcpy(Name, prop.ModifierStr[M_YMAX]);
				  lstrcpy(prop.ModifierStr[M_YMAX], prop.ModifierStr[M_YMIN]);
				  lstrcpy(prop.ModifierStr[M_YMIN], Name);
				  SetDlgItemText(hDlg, ID_YMIN, prop.ModifierStr[M_YMIN]);
				  SetDlgItemText(hDlg, ID_YMAX, prop.ModifierStr[M_YMAX]);
				}
				else
				  break;
			 }
			 if ((prop.Modifier[M_XMIN] == prop.Modifier[M_XMAX]) ||
				  (prop.Modifier[M_YMIN] == prop.Modifier[M_YMAX]))
			 {
				MessageBeep(MB_ICONEXCLAMATION);
				MessageBox(hDlg, "Your Minimum and Maximums CANNOT equal!", "Range Error", MB_OK | MB_ICONEXCLAMATION);
				break;
			 }
			 if ((prop.Modifier[M_XTICK] < 0) || (prop.Modifier[M_YTICK] < 0))
			 {
				int lcv, pos;
				char Name[NAMELEN];
				MessageBeep(MB_ICONEXCLAMATION);
				MessageBox(hDlg, "Your Tick Ranges must be greater than zero.  Press [Ok] to correct this.", "Range Error", MB_OK | MB_ICONEXCLAMATION);
				if (prop.Modifier[M_YTICK] < 0)
				  prop.Modifier[M_YTICK] = 0 - prop.Modifier[M_YTICK];
				if (prop.Modifier[M_XTICK] < 0)
				  prop.Modifier[M_XTICK] = 0 - prop.Modifier[M_XTICK];
				pos = 0;
				for (lcv = 0; lcv <= NAMELEN - 1; lcv++)
				{
				  if (prop.ModifierStr[M_YTICK][lcv] != '-')
				  {
					 Name[pos] = prop.ModifierStr[M_YTICK][lcv];
					 pos++;
				  }
				}
				lstrcpy(prop.ModifierStr[M_YTICK], Name);
				pos = 0;
				for (lcv = 0; lcv <= NAMELEN - 1; lcv++)
				{
				  if (prop.ModifierStr[M_XTICK][lcv] != '-')
				  {
					 Name[pos] = prop.ModifierStr[M_XTICK][lcv];
					 pos++;
				  }
				}
				lstrcpy(prop.ModifierStr[M_XTICK], Name);
				SetDlgItemText(hDlg, ID_XTICK, prop.ModifierStr[M_XTICK]);
				SetDlgItemText(hDlg, ID_YTICK, prop.ModifierStr[M_YTICK]);
			 }

			 // Put the properties into the main gp structure
			 CopyMemory(&gp, &prop, sizeof(GridProperties));
			 GraphUpdated = TRUE;

			 // If the user pressed Ok, then exit the dialog box;
			 // otherwise, invalidate the main window's rectangle
			 if (LOWORD(wParam) == IDOK)
				EndDialog(hDlg, 0);
			 else
				InvalidateRect(GetParent(hDlg), NULL, TRUE);
			 break;
		  }
		}
		return 0;
	 }
  }
  return FALSE;
}

void ChangeMenuFlag(int MenuItem, BOOL *Flag, HWND hWnd)
{
  if (*Flag)
  {
	 CheckMenuItem(hMenu, MenuItem, MF_UNCHECKED);
	 *Flag = FALSE;
  }
  else
  {
	 CheckMenuItem(hMenu, MenuItem, MF_CHECKED);
	 *Flag = TRUE;
  }
  InvalidateRect(hWnd, NULL, TRUE);
}

void CalculateModifiers(GridProperties *g)
{
  int lcv;
  double Multiplier;

  for (lcv = 0; lcv <= NUM_MODIFIERS - 1; lcv++)
  {
	 if ((g->ModifierStr[lcv][strlen(g->ModifierStr[lcv]) - 1] == 'p') ||
		 (g->ModifierStr[lcv][strlen(g->ModifierStr[lcv]) - 1] == 'P'))
	 {
		if ((g->ModifierStr[lcv][0] == '-') && (lstrlen(g->ModifierStr[lcv]) == 2))
		{
		  lstrcpy(g->ModifierStr[lcv], "-1p");
		  Multiplier = M_PI;
		}
		else
		  Multiplier = M_PI;
	 }
	 else
		Multiplier = 1;
	 if ((g->ModifierStr[lcv][0] == 'p') ||
		 (g->ModifierStr[lcv][0] == 'P'))
		g->Modifier[lcv] = M_PI;
	 else
	 {
		g->Modifier[lcv] = atof(g->ModifierStr[lcv]);
		g->Modifier[lcv] = g->Modifier[lcv] * Multiplier;
	 }
  }
}

